import _extends from "@babel/runtime/helpers/extends";
import remark from 'remark';
import { getTransformValue } from './transform';
import { FUNNAME_PREFIX } from '../';

/**
 * Creates an object containing the parameters of the current URL.
 *
 * ```js
 * getURLParameters('name=Adam&surname=Smith');
 * // 👉 {name: 'Adam', surname: 'Smith'}
 * ```
 * @param url `name=Adam&surname=Smith`
 * @returns
 */
export var getURLParameters = url => (url.match(/([^?=&]+)(=([^&]*))/g) || []).reduce((a, v) => (a[v.slice(0, v.indexOf('='))] = v.slice(v.indexOf('=') + 1), a), {});
/** 转换 代码*/
export var getProcessor = source => {
  try {
    var child = remark.parse(source);
    return child.children;
  } catch (err) {
    console.warn(err);
  }
};

/**
 * ```js
 * 'mdx:preview' => ''  // Empty
 * 'mdx:preview:demo12' => 'demo12' // return meta id => 'demo12'
 * ```
 * @param meta string
 * @returns string?
 */
export var getMetaId = function getMetaId(meta) {
  if (meta === void 0) {
    meta = '';
  }
  var [metaRaw = ''] = /mdx:(.[\w|:]+)/i.exec(meta) || [];
  return metaRaw.replace(/^mdx:preview:?/, '');
};

/**
 * ```js
 * isMeta('mdx:preview') => true
 * isMeta('mdx:preview:demo12') => true
 * isMeta('mdx:preview--demo12') => false
 * ```
 * @param meta
 * @returns boolean
 */
export var isMeta = function isMeta(meta) {
  if (meta === void 0) {
    meta = '';
  }
  return meta && meta.includes('mdx:preview');
};

/** 获取需要渲染的代码块 **/
export function getCodeBlock(child, opts, resourcePath) {
  if (opts === void 0) {
    opts = {};
  }
  var {
    lang = ['jsx', 'tsx']
  } = opts;
  // 获取渲染部分
  var codeBlock = {};
  child.forEach(item => {
    if (item && item.type === 'code' && lang.includes(item.lang)) {
      var line = item.position.start.line;
      var metaId = getMetaId(item.meta);
      if (isMeta(item.meta)) {
        var name = metaId || line;
        var funName = resourcePath + "." + FUNNAME_PREFIX + name;
        var returnCode = getTransformValue(item.value, funName + "." + item.lang, opts);
        codeBlock[name] = {
          name,
          meta: getURLParameters(item.meta),
          code: returnCode,
          language: item.lang,
          value: item.value
        };
      }
    }
  });
  return codeBlock;
}

/**
 * `mdCodeModulesLoader` method for adding `markdown-react-code-preview-loader` to webpack config.
 * @param {webpack.Configuration} config webpack config
 * @param {string[]} lang Parsing language
 * @param {Options} option Loader Options
 * @returns {webpack.Configuration}
 */
export var mdCodeModulesLoader = function mdCodeModulesLoader(config, lang, option) {
  if (option === void 0) {
    option = {};
  }
  config.module.rules.forEach(ruleItem => {
    if (typeof ruleItem === 'object') {
      if (ruleItem.oneOf) {
        ruleItem.oneOf.unshift({
          test: /.md$/,
          use: [{
            loader: 'markdown-react-code-preview-loader',
            options: _extends({
              lang
            }, option)
          }]
        });
      }
    }
  });
  return config;
};
//# sourceMappingURL=index.js.map